# Load blender specific modules
import bpy
from bpy import context
from bpy.props import *
from bpy.types import Menu, Panel, UIList, Operator, AddonPreferences
from bl_ui.properties_data_modifier import DATA_PT_modifiers

# load an assortment of shit
import sys
import os
import tempfile
import json

# Find and load the .stl plugin modules (I'm using this to import .stl files)
stlpath = bpy.utils.script_paths(subdir='addons')[0] + "\\io_mesh_stl\\"
sys.path.append(stlpath)
import stl_utils
import blender_utils
from mathutils import Matrix


bl_info = {
    'name': 'OpenJSCAD Bridge',
    'author': 'falldeaf',
    'version': (0, 1, 0),
    'blender': (2, 74, 0),
    'location': 'View3D > UI panel > OpenJSCAD',
    'description': 'Create parametric models based on OpenJSCAD ',
    'warning': 'This plugin requires an install of nodeJS and OpenJSCAD: https://github.com/Spiritdude/OpenJSCAD.org/wiki/User-Guide#local-installation-offline',
    'wiki_url': 'http://falldeaf.com/',
    'tracker_url': '',
    'support': 'COMMUNITY',
    'category': 'Add Mesh'}

param_json = []
param_name = ""


class openjscadAddonPreferences(AddonPreferences):
    bl_idname = __name__

    node_path = bpy.props.StringProperty(
        name="openJSCAD script Path",
        default="",
        description="Ay, where ya openjscad script at, dog?!",
        subtype='DIR_PATH')

    conf_path = bpy.props.StringProperty(
        name=".jscad files Path",
        default="",
        description="Yo, where yo .jscad files at?!",
        subtype='DIR_PATH')

    def draw(self, context):
        layout = self.layout
        layout.label(text="This is a preferences view for our addon")
        layout.prop(self, "node_path")
        layout.prop(self, "conf_path")


class OBJECT_OT_addon_prefs_openjscad(Operator):
    """Display example preferences"""
    bl_idname = "object.addon_prefs_openjscad"
    bl_label = "Addon Preferences openjscad"
    bl_options = {'REGISTER', 'UNDO'}

    global addon_prefs

    def execute(self, context):
        user_preferences = context.user_preferences
        addon_prefs = user_preferences.addons[__name__].preferences

        info = ("Path: %s, Number: %d, Boolean %r" %
                (addon_prefs.conf_path, addon_prefs.node_path))

        self.report({'INFO'}, info)
        print(info)

        return {'FINISHED'}


def newProps(self, context):
    global param_json
    global param_name
    global openJSCADPanel

    param_json = []
    param_name = ""

    user_preferences = context.user_preferences
    addon_prefs = user_preferences.addons[__name__].preferences

    param_name = bpy.context.scene.openjscadfiles

    print(addon_prefs.conf_path + bpy.context.scene.openjscadfiles)
    with open(addon_prefs.conf_path + bpy.context.scene.openjscadfiles, 'r') as myfile:
        data = myfile.read().replace('\n', '')

    data = data.replace('\n', ' ').replace('\r', '')
    marker = data.find("getParameterDefinitions")
    if marker == -1:
        # no params!
        print("no params")
    else:
        marker_end = data.find("function", marker)
        functionchunk = data[marker:marker_end]
        jsonbegin = functionchunk.find("[")
        jsonend = functionchunk.rfind("]")
        readytoparsestring = functionchunk[jsonbegin:jsonend+1]
        readytoparsestring = readytoparsestring.replace("'", '"')

        try:
            param_json = json.loads(readytoparsestring)
        except:
            print("Couldn't parse params!")
            self.report({'ERROR'}, "Cannot parse .jscad params")

        print("param panel!")
        if param_name != "":
            unregister2()
        initParams(bpy.context.scene)
        register2()


def newFiles(self, context):
    bpy.props.EnumProperty(items=fileListFill)


def fileListFill(self, context):
    user_preferences = context.user_preferences
    addon_prefs = user_preferences.addons[__name__].preferences

    list = []
    for dirname, dirnames, filenames in os.walk(addon_prefs.conf_path):
        for filename in filenames:
            if filename.endswith(".jscad"):
                list.append((filename, filename, "noo"))

    return list


def initSceneProperties(scn):
    bpy.types.Scene.openjscadfiles = EnumProperty(
        items=fileListFill,
        name="files",
        update=newProps)
    scn['openjscadfiles'] = 2


class OBJECT_OT_loadfiles(bpy.types.Operator):
    bl_idname = "idname.load_options"
    bl_label = "Load Files"

    def execute(self, context):
        scn = context.scene
        print("load files")
        initSceneProperties(scn)
        newFiles(self, context)

        return {'FINISHED'}


class openJSCADPanel(bpy.types.Panel):
    bl_label = "OpenJSCAD"
    bl_idname = "OBJECT_PT_OpenJSCAD"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_category = "OpenJSCAD"

    def draw(self, context):
        global prefset
        layout = self.layout
        scn = context.scene

        layout.operator("idname.load_options")
        layout.prop(scn, 'openjscadfiles')


class openJSCADparamsPanel(bpy.types.Panel):
    bl_label = "parameters"
    bl_idname = "OBJECT_PT_OJSCparams"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_category = "OpenJSCAD"

    def draw(self, context):
        layout = self.layout
        scn = context.scene

        dynLayout(layout, scn)


def dynLayout(layout, scn):
    global param_json
    global param_name

    layout.label(param_name)

    for rows in param_json:
        exec('layout.prop(scn, "' + rows['name'] + '")')

    layout.operator("idname.create_mesh")


class OBJECT_OT_createmesh(bpy.types.Operator):
    bl_idname = "idname.create_mesh"
    bl_label = "Create mesh"

    def execute(self, context):
        scn = context.scene
        print("get props and do cmd line shit!")
        makeMesh(scn)

        return {'FINISHED'}


def initParams(scn):
    global param_json

    for rows in param_json:
        args = ""
        print("\nRows:")
        if rows['type'] == 'int':
            if 'caption' in rows:
                args = args + 'description = "' + rows['caption'] + '", '
            args = args + 'name = "' + rows['name'] + '"'
            print("\n" + args)
            exec('bpy.types.Scene.' + rows['name'] + ' = IntProperty(' + args + ')')
            if rows['default']:
                scn[rows['name']] = int(rows['default'])

        elif rows['type'] == 'float':
            if 'caption' in rows:
                args = args + 'description = "' + rows['caption'] + '", '
            if 'default' in rows:
                args = args + 'default = float(' + str(rows['default']) + '), '
            args = args + 'name = "' + rows['name'] + '"'
            print("\n" + args)
            exec('bpy.types.Scene.' + rows['name'] + ' = FloatProperty(' + args + ')')
            if rows['default']:
                scn[rows['name']] = float(rows['default'])

        elif rows['type'] == 'choice':
            if 'caption' in rows:
                args = args + 'description = "' + rows['caption'] + '", '
            if 'default' in rows:
                args = args + 'default = "' + rows['default'] + '", '
            args = args + 'items = ['
            for item in rows['values']:
                args = args + '("' + item + '", "' + item + '", "' + item + '")'
                args = args + ','
            args = args[:-1]
            args = args + '], '
            # print('items: [%s]' % ', '.join(map(str, rows['values'])))
            args = args + 'name = "' + rows['name'] + '"'
            print("\n" + args)
            exec('bpy.types.Scene.' + rows['name'] + ' = EnumProperty(' + args + ')')

        elif rows['type'] == 'text':
            if 'caption' in rows:
                args = args + 'description = "' + rows['caption'] + '", '
            if 'default' in rows:
                args = args + 'default = "' + rows['default'] + '", '
            # print("\nRows:" + rows['values'])
            args = args + 'name = "' + rows['name'] + '"'
            print("\n" + args)
            exec('bpy.types.Scene.' + rows['name'] + ' = StringProperty(' + args + ')')


def register():
    bpy.utils.register_class(OBJECT_OT_addon_prefs_openjscad)
    bpy.utils.register_class(openjscadAddonPreferences)
    bpy.utils.register_class(openJSCADPanel)
    bpy.utils.register_module(__name__)


def unregister():
    bpy.utils.unregister_class(OBJECT_OT_addon_prefs_openjscad)
    bpy.utils.unregister_class(openjscadAddonPreferences)
    bpy.utils.unregister_class(openJSCADPanel)
    bpy.utils.unregister_module(__name__)
    unregister2()


def register2():
    bpy.utils.register_class(openJSCADparamsPanel)


def unregister2():
    bpy.utils.unregister_class(openJSCADparamsPanel)

if __name__ == "__main__":
    register()


def makeMesh(scn):
    global param_json
    global param_name

    user_preferences = context.user_preferences
    addon_prefs = user_preferences.addons[__name__].preferences

    # tempfile location
    temp = tempfile.gettempdir() + "temp.stl"

    # Start the cmd line string with the program openjscad with the selected .jscad file
    cmd = "node " + addon_prefs.node_path + "openjscad " + addon_prefs.conf_path + bpy.context.scene.openjscadfiles + " "

    # add all the params as flags
    for rows in param_json:
        key = rows['name']
        cmd = cmd + " --" + key + "=" + str(getattr(bpy.context.scene, key))

    # add the -output command (Save .stl to temp dir)
    cmd = cmd + " -o " + temp

    print(cmd)
    os.chdir(addon_prefs.node_path)
    os.system(cmd)
    print("cmd run")

    if bpy.ops.object.mode_set.poll():
        bpy.ops.object.mode_set(mode='OBJECT')

    if bpy.ops.object.select_all.poll():
        bpy.ops.object.select_all(action='DESELECT')

    tris, pts = stl_utils.read_stl(temp)
    blender_utils.create_and_link_mesh(param_name, tris, pts, Matrix())
